package com.example.tcc.api;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.example.tcc.document.models.User;
import com.example.tcc.service.UserService;
import com.example.tcc.service.exception.*;
import com.example.tcc.service.utils.Tools;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

import static com.example.tcc.service.utils.Constants.ERROR_MESSAGE;
import static com.example.tcc.service.utils.Constants.MISSING_ARGUMENTS;

@RestController
@RequestMapping(produces = MediaType.APPLICATION_JSON_VALUE)
public class UserController {

	@Autowired
	UserService userService;

	@GetMapping("public/token/valid")
	public ResponseEntity<Object> isTokenValid(HttpServletRequest request) {
		Map<String, Boolean> payload = new HashMap<>();
		payload.put("valid", userService.isTokenValid(request));
		return ResponseEntity.ok(payload);
	}

	@GetMapping("public/token/refresh")
	public void refreshToken(HttpServletRequest request, HttpServletResponse response) {
		try {
			userService.refreshToken(request, response);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@PostMapping("public/user/save")
	public ResponseEntity<Object> saveUser(@RequestBody User user) {
		try {
			userService.saveUser(user);
		} catch (RepeatedUsernameException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		} catch (MissingArgumentsException e) {
			Map<String, Object> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			error.put(MISSING_ARGUMENTS, e.getArguments());
			return ResponseEntity.badRequest().body(error);
		}
		return ResponseEntity.ok().build();
	}

	@GetMapping("public/user/{username}")
	public ResponseEntity<Object> findUserProfile(@PathVariable String username) {
		try {
			return ResponseEntity.ok(userService.findUserProfile(username));
		} catch (UserNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@GetMapping("public/user/{username}/work-list")
	public ResponseEntity<Object> findUserWorkList(@PathVariable String username) {
		try {
			return ResponseEntity.ok(userService.findUserWorkList(username));
		} catch (UserNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		} catch (EmptyWorkListException e) {
			return ResponseEntity.noContent().build();
		}
	}

	@GetMapping("public/user/{username}/work-list/{workId}")
	public ResponseEntity<Object> findUserWorkListById(@PathVariable String username, @PathVariable String workId) {
		try {
			return ResponseEntity.ok(userService.findUserWorkListById(username, workId));
		} catch (UserNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		} catch (WorkNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@PutMapping("api/user/edit")
	public ResponseEntity<Object> updateUserProfile(HttpServletRequest request, @RequestBody User user) {
		DecodedJWT jwt = Tools.decryptToken(request);
		userService.updateUserProfile(jwt.getSubject(), user);
		return ResponseEntity.ok().build();
	}

	@GetMapping("admin/user/list")
	public ResponseEntity<Object> findAll() {
		return ResponseEntity.ok(userService.findAll());
	}
}
