package com.example.tcc.api;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.example.tcc.document.models.Work;
import com.example.tcc.document.models.WorkUser;
import com.example.tcc.service.WorkService;
import com.example.tcc.service.exception.*;
import com.example.tcc.service.utils.Tools;
import org.apache.coyote.Response;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.example.tcc.service.utils.Constants.ERROR_MESSAGE;
import static com.example.tcc.service.utils.Constants.MISSING_ARGUMENTS;

@RestController
@RequestMapping(produces = MediaType.APPLICATION_JSON_VALUE)
public class WorkController {

	@Autowired
	WorkService workService;

	@GetMapping("public/work/list")
	public ResponseEntity<Object> findAll(@RequestParam Integer page) {
		return ResponseEntity.ok(workService.findAll(page));
	}

	@GetMapping("public/work")
	public ResponseEntity<Object> findWorkById(@RequestParam String id) {
		try {
			return ResponseEntity.ok(workService.findWork(id));
		} catch (WorkNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@GetMapping("public/work/famous")
	public ResponseEntity<Object> findMostFamous(@RequestParam String type) {
		return ResponseEntity.ok(workService.findMostFamousWorks(type));
	}

	@GetMapping("public/work/by-tags")
	public ResponseEntity<Object> findByTags(@RequestParam String type) {
		return ResponseEntity.ok(workService.findByTags(type));
	}

	@GetMapping("public/work/search")
	public ResponseEntity<Object> searchWorks(
			@RequestParam String type,
			@RequestParam String input,
			@RequestParam Integer page
	){
		return ResponseEntity.ok(workService.searchWorks(type, input, page));
	}

	@GetMapping("public/work/tags")
	public ResponseEntity<Object> findTags() {
		return ResponseEntity.ok(workService.findAllTags());
	}

	@GetMapping("api/work/recommendations")
	public ResponseEntity<Object> findRecommendations(HttpServletRequest request, @RequestParam String type) {
		DecodedJWT jwt = Tools.decryptToken(request);
		try {
			return ResponseEntity.ok(workService.findRecommendationsForUser(jwt.getSubject(), type));
		} catch (UserNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		} catch (EmptyWorkListException e) {
			return ResponseEntity.noContent().build();
		}
	}

	@PostMapping("api/work/add-work-to-list")
	public ResponseEntity<Object> saveWorkToLoggedUserList(HttpServletRequest request, @RequestBody WorkUser workUser) {
		DecodedJWT jwt = Tools.decryptToken(request);
		try {
			workService.saveWorkToLoggedUserList(jwt.getSubject(), workUser);
			return ResponseEntity.ok().build();
		} catch (WorkNotFoundException | UserNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@PostMapping("admin/work/save")
	public ResponseEntity<Object> saveWork(@RequestBody Work work) {
		try {
			workService.saveWork(work);
			return ResponseEntity.ok().build();
		} catch (InvalidStatusException | RepeatedWorkException | MissingRoleException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		} catch (MissingArgumentsException e) {
			Map<String, Object> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			error.put(MISSING_ARGUMENTS, e.getArguments());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@PutMapping("admin/work/avg-scores")
	public ResponseEntity<Object> calculateWorksAverageScore() {
		workService.calculateWorksAverageScore();
		return ResponseEntity.ok().build();
	}

	@PostMapping("admin/work/save/csv")
	public ResponseEntity<Object> insertWorksByCsv(@RequestParam("file") MultipartFile file) {
		try {
			workService.insertWorksByCsv(file);
			return ResponseEntity.ok().build();
		} catch (IOException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}

	@DeleteMapping("admin/work/delete")
	public ResponseEntity<Object> deleteWork(@RequestParam String id) {
		try {
			workService.deleteWork(id);
			return ResponseEntity.noContent().build();
		} catch (WorkNotFoundException e) {
			Map<String, String> error = new HashMap<>();
			error.put(ERROR_MESSAGE, e.getMessage());
			return ResponseEntity.badRequest().body(error);
		}
	}
}
