package com.example.tcc.service;

import com.example.tcc.document.models.Person;
import com.example.tcc.document.models.PersonWork;
import com.example.tcc.document.models.Work;
import com.example.tcc.service.exception.*;
import com.example.tcc.service.utils.Tools;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.example.tcc.service.utils.Constants.*;

@Service
public class PersonService extends AbstractService {

//	private Person findByNameWithFields(String name, String... fields) throws PersonNotFoundException {
//
//	}

	public Person findPerson(String name) throws PersonNotFoundException {
		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_NAME).is(name));
		Person person = db.findOne(query, Person.class);
		if(Objects.isNull(person)) throw new PersonNotFoundException("Pessoa com nome " + name + " não encontrada");
		return person;
	}

	public List<Person> searchPeople(String input, Integer page){
		log.info("Initializing people search with input: "+input);
		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_NAME).regex(input, "i"))
				.with(PageRequest.of(page, 20, Sort.by(FIELD_NAME).ascending()));
		return db.find(query, Person.class);
	}

	public List<Person> findAll(Integer page) {
		Query query = new Query()
				.with(PageRequest.of(page, 20, Sort.by(FIELD_NAME).ascending()));
		return db.find(query, Person.class);
	}

	public void savePerson(Person newPerson) throws RepeatedPersonException, MissingArgumentsException, WorkNotFoundException, MissingRoleException {
		if(Objects.nonNull(newPerson.getWorkedOn())) {
			for(PersonWork personWork : newPerson.getWorkedOn()) {
				if(Objects.isNull(personWork.getRole()))
					throw new MissingRoleException("Pessoa na obra " + personWork.getWork().getId() + " não possui um cargo!");
				Work workDb = db.findOne(new Query().addCriteria(Criteria.where(FIELD_ID).is(personWork.getWork().getId())), Work.class);
				if(Objects.nonNull(workDb)) {
					personWork.getWork().setName(workDb.getName());
					personWork.getWork().setCover(workDb.getCover());
					personWork.getWork().setType(workDb.getType());
				} else throw new WorkNotFoundException("Obra não encontrada: " + personWork.getWork().getId());
			}
		}

		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_ID).is(newPerson.getId()));
		Person repeatedPerson = db.findOne(query, Person.class);

		if(Objects.isNull(repeatedPerson)) {
			validatePerson(newPerson);
			log.info("Salvando nova pessoa: " + newPerson.getName());
		} else {
			Tools.copyNonNullProperties(repeatedPerson, newPerson);
			if(Objects.nonNull(newPerson.getWorkedOn())
					&& !newPerson.getWorkedOn().isEmpty())
				repeatedPerson.setWorkedOn(newPerson.getWorkedOn());
			newPerson = repeatedPerson;
			log.info("Atualizando pessoa: " + newPerson.getName());
		}

		newPerson = db.save(newPerson);
		savePeopleOnWorks(newPerson);
	}

	private void savePeopleOnWorks(Person newPerson) {
		Update update = new Update().pull(FIELD_PEOPLE, Query.query(Criteria.where("person._id").is(newPerson.getId())));
		db.updateMulti(new Query(), update, Work.class);
		if(Objects.nonNull(newPerson.getWorkedOn()))
			for(PersonWork personWork : newPerson.getWorkedOn()) {
				Query query = new Query().addCriteria(Criteria
						.where(FIELD_ID).is(personWork.getWork().getId()));
				PersonWork newPersonWork = new PersonWork();
				newPersonWork.setRole(personWork.getRole());
				newPersonWork.setIsMain(personWork.getIsMain());
				newPersonWork.setWork(null);
				newPersonWork.setPerson(new Person(newPerson.getId(),
						newPerson.getName(),
						newPerson.getPhoto()));
				update = new Update().addToSet(FIELD_PEOPLE, newPersonWork);
				db.updateFirst(query, update, Work.class);
			}
	}

	private void validatePerson(Person newPerson) throws RepeatedPersonException, MissingArgumentsException {
		Query query = new Query().addCriteria(Criteria.where(FIELD_NAME).is(newPerson.getName()));
		Person repeatedPerson = db.findOne(query, Person.class);
		if(Objects.nonNull(repeatedPerson))
			throw new RepeatedPersonException("Erro ao salvar nova pessoa: Pessoa com nome " + newPerson.getName() + " já existe!");

		ArrayList<String> missingArguments = new ArrayList<>();
		if(Objects.isNull(newPerson.getName())) missingArguments.add(FIELD_NAME);
		if(!missingArguments.isEmpty()) {
			log.info("Erro ao salvar nova obra: Argumentos faltando");
			throw new MissingArgumentsException("Erro ao salvar usuário: Argumentos faltando", missingArguments);
		}
	}

	public void deletePerson(String id) throws PersonNotFoundException {
		log.info("Deletando pessoa com id " + id);
		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_ID).is(id));
		query.fields().include(FIELD_ID);
		Person person = db.findOne(query, Person.class);
		if(person == null) throw new PersonNotFoundException("Pessoa com id " + id + " não encontrada");

		List<Work> works = db.find(Query.query(Criteria.where("people.person._id").is(id)), Work.class);
		for (Work work : works) {
			work.getPeople().removeIf(pw -> pw.getPerson().getId().equals(id));
			db.save(work);
		}

		db.remove(person);
	}
}
