package com.example.tcc.service;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.example.tcc.document.models.Admin;
import com.example.tcc.document.models.Statistics;
import com.example.tcc.document.models.User;
import com.example.tcc.document.models.WorkUser;
import com.example.tcc.service.exception.*;
import com.example.tcc.service.utils.Tools;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.example.tcc.service.utils.Constants.*;
import static org.springframework.http.HttpHeaders.AUTHORIZATION;
import static org.springframework.http.HttpStatus.FORBIDDEN;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@Service
public class UserService extends AbstractService implements UserDetailsService {

	@Autowired
	private PasswordEncoder passwordEncoder;
	@Autowired
	private AdminService adminService;

	private User findByUsernameWithFields(String username, String... fields) throws UserNotFoundException {
		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_USERNAME).is(username))
				.fields().exclude(FIELD_ID)
				.include(fields);
		User user = db.findOne(query, User.class);
		if(Objects.isNull(user)) throw new UserNotFoundException("Usuário com username " + username + " não encontrado");
		return user;
	}

	private String findRandomFavoriteWork(String username) {
		List<WorkUser> workList;
		try {
			workList = findUserWorkList(username);
		} catch (UserNotFoundException | EmptyWorkListException e) {
			return null;
		}
		workList.removeIf(work -> Objects.isNull(work.getScore()) || work.getScore() < 4);

		Random r = new Random();
		WorkUser randomTopWork = workList.get(r.nextInt(workList.size()));
		return randomTopWork.getWork().getCover();
	}

	public User findUserProfile(String username) throws UserNotFoundException {
		User user = findByUsernameWithFields(username,
				FIELD_NAME,
				FIELD_USERNAME,
				FIELD_EMAIL,
				FIELD_DESCRIPTION,
				FIELD_PHOTO,
				FIELD_BIRTH_DATE,
				FIELD_COVER_OPTION,
				FIELD_PROFILE_COVER,
				FIELD_WORK_LIST);

		if(COVER_OPTION_CYCLE_FAVORITES.equals(user.getCoverOption()))
			user.setProfileCover(findRandomFavoriteWork(username));

		if(user.getWorkList() == null || user.getWorkList().isEmpty()) return user;

		// Statistics
		Map<String, Statistics> statisticsByType = new HashMap<>();
		Map<String, Double> totalScoreByType = new HashMap<>();
		Map<String, Integer> sizeByType = new HashMap<>();
		for(String type : TYPES_LIST) {
			statisticsByType.put(type, new Statistics(type));
			totalScoreByType.put(type, 0D);
			sizeByType.put(type, 0);
		}
		Statistics statisticsAny = statisticsByType.get(TYPE_ANY);

		for(WorkUser workUser : user.getWorkList()) {
			String type = workUser.getWork().getType();
			Statistics statistics = statisticsByType.get(type);
			if(Objects.isNull(statistics)) continue;

			Map<String, Integer> worksByStatus = statistics.getWorksByStatus();
			worksByStatus.replace(workUser.getStatus(), worksByStatus.get(workUser.getStatus()) + 1);

			Map<String, Integer> worksByStatusAny = statisticsAny.getWorksByStatus();
			worksByStatusAny.replace(workUser.getStatus(), worksByStatusAny.get(workUser.getStatus()) + 1);

			if(Objects.nonNull(workUser.getProgress()))
				statistics.setTotalProgress(statistics.getTotalProgress() + workUser.getProgress());

			if(Objects.nonNull(workUser.getScore())) {
				totalScoreByType.replace(type, totalScoreByType.get(type) + workUser.getScore());
				sizeByType.replace(type, sizeByType.get(type) + 1);
				totalScoreByType.replace(TYPE_ANY, totalScoreByType.get(TYPE_ANY) + workUser.getScore());
				sizeByType.replace(TYPE_ANY, sizeByType.get(TYPE_ANY) + 1);
			}
		}
		for(Map.Entry<String, Double> totalScore : totalScoreByType.entrySet()) {
			if(sizeByType.get(totalScore.getKey()) != 0D)
				statisticsByType.get(totalScore.getKey()).setAverageScore(totalScore.getValue() / sizeByType.get(totalScore.getKey()));
		}

		List<Statistics> statisticsList = new ArrayList<>();
		for(Map.Entry<String, Statistics> statistics : statisticsByType.entrySet()) {
			statisticsList.add(statistics.getValue());
		}
		user.setWorkList(null);
		user.setStatisticsList(statisticsList);

		return user;
	}

	public List<WorkUser> findUserWorkList(String username) throws UserNotFoundException, EmptyWorkListException {
		User user = findByUsernameWithFields(username, FIELD_WORK_LIST);
		if(Objects.isNull(user.getWorkList()) ||
				user.getWorkList().isEmpty())
			throw new EmptyWorkListException("Lista de obras do usuário atual vazia");
		return user.getWorkList();
	}

	public WorkUser findUserWorkListById(String username, String workId) throws UserNotFoundException, WorkNotFoundException {
		Optional<WorkUser> workUser = findByUsernameWithFields(username, FIELD_WORK_LIST)
				.getWorkList()
				.stream()
				.filter(wu -> workId.equals(wu.getWork().getId()))
				.findFirst();

		if(workUser.isEmpty()) throw new WorkNotFoundException("Obra não encontrada na lista de "+username);
		else {
			workUser.get().setWork(null);
			return workUser.get();
		}
	}

	public void saveUser(User user) throws MissingArgumentsException, RepeatedUsernameException {
		ArrayList<String> missingArguments = new ArrayList<>();
		if(Objects.isNull(user.getName())) missingArguments.add(FIELD_NAME);
		if(Objects.isNull(user.getUsername())) missingArguments.add(FIELD_USERNAME);
		if(Objects.isNull(user.getPassword())) missingArguments.add(FIELD_PASSWORD);
		if(Objects.isNull(user.getEmail())) missingArguments.add(FIELD_EMAIL);
		if(Objects.isNull(user.getBirthDate())) missingArguments.add(FIELD_BIRTH_DATE);
		if(!missingArguments.isEmpty()) {
			log.info("Erro ao salvar usuário: Argumentos faltando");
			throw new MissingArgumentsException("Erro ao salvar usuário: Argumentos faltando", missingArguments);
		}

		try {
			findByUsernameWithFields(user.getUsername(), FIELD_USERNAME);
			log.error("Erro ao salvar usuário: Usuário com username " + user.getUsername() + " já existe");
			throw new RepeatedUsernameException("Erro ao salvar usuário: Usuário com username " + user.getUsername() + " já existe");
		} catch (UserNotFoundException e) {
			Query query = new Query();
			query.addCriteria(Criteria.where(FIELD_EMAIL).is(user.getEmail())).fields().exclude(FIELD_ID);
			User repeatedUser = db.findOne(query, User.class);
			if(repeatedUser != null){
				log.error("Erro ao salvar usuário: Usuário com email " + user.getEmail() + " já existe");
				throw new RepeatedUsernameException("Erro ao salvar usuário: Usuário com email " + user.getEmail() + " já existe");
			}

			log.info("Salvando novo usuário " + user.getUsername());
			user.setCoverOption(COVER_OPTION_CYCLE_FAVORITES);
			user.setPassword(passwordEncoder.encode(user.getPassword()));
			db.insert(user);
		}
	}

	public void updateUserProfile(String username, User user) {
		Query query = new Query().addCriteria(Criteria.where(FIELD_USERNAME).is(username));
		Update update = new Update();
		if(Objects.nonNull(user.getCoverOption())) {
			update.set(FIELD_COVER_OPTION, user.getCoverOption());
			if(COVER_OPTION_CHOOSE.equals(user.getCoverOption()) &&
					Objects.nonNull(user.getProfileCover()))
				update.set(FIELD_PROFILE_COVER, user.getProfileCover());
		}
		if(Objects.nonNull(user.getPhoto())) update.set(FIELD_PHOTO, user.getPhoto());
		if(Objects.nonNull(user.getDescription())) update.set(FIELD_DESCRIPTION, user.getDescription());
		if(Objects.nonNull(user.getName())) update.set(FIELD_NAME, user.getName());
		db.updateFirst(query, update, User.class);
	}

	public List<User> findAll() {
		return db.findAll(User.class);
	}

	// Security flows
	@Override
	public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
		try {
			User user = findByUsernameWithFields(username, FIELD_USERNAME, FIELD_PASSWORD);
			return new org.springframework.security.core.userdetails.User(user.getUsername(), user.getPassword(), new ArrayList<>());
		} catch (UserNotFoundException e) {
			Admin admin = adminService.findByUsername(username);
			if(Objects.nonNull(admin)) {
				Collection<SimpleGrantedAuthority> authorities = Collections.singletonList(new SimpleGrantedAuthority(ROLE_ADMIN));
				return new org.springframework.security.core.userdetails.User(admin.getUsername(), admin.getPassword(), authorities);
			} else
				throw new UsernameNotFoundException("Usuário não encontrado");
		}
	}

	public boolean isTokenValid(HttpServletRequest request) {
		DecodedJWT jwt = Tools.decryptToken(request);
		return new Date().before(jwt.getExpiresAt());
	}

	public void refreshToken(HttpServletRequest request, HttpServletResponse response) throws IOException, MissingArgumentsException {
		String authorizationHeader = request.getHeader(AUTHORIZATION);
		if(Objects.nonNull(authorizationHeader) && authorizationHeader.startsWith("Bearer ")) {
			try {
				String refreshToken = authorizationHeader.substring("Bearer ".length());
				Algorithm algorithm = Algorithm.HMAC256("secret".getBytes());
				JWTVerifier verifier = JWT.require(algorithm).build();
				DecodedJWT decodedJWT = verifier.verify(refreshToken);
				String username = decodedJWT.getSubject();

				Map<String, String> tokens = new HashMap<>();
				String accessToken;
				try {
					User user = findByUsernameWithFields(username, FIELD_USERNAME);
					accessToken = JWT.create()
							.withSubject(user.getUsername())
							.withExpiresAt(new Date(System.currentTimeMillis() + TimeUnit.HOURS.toMillis(1)))
							.withIssuer(request.getRequestURL().toString())
							.withClaim("roles", new ArrayList<>())
							.sign(algorithm);
					tokens.put("role", ROLE_USER);
				} catch (UserNotFoundException e) {
					Admin admin = adminService.findByUsername(username);
					accessToken = JWT.create()
							.withSubject(admin.getUsername())
							.withExpiresAt(new Date(System.currentTimeMillis() + TimeUnit.HOURS.toMillis(1)))
							.withIssuer(request.getRequestURL().toString())
							.withClaim("roles", Collections.singletonList(ROLE_ADMIN))
							.sign(algorithm);
					tokens.put("role", ROLE_ADMIN);
				}

				tokens.put("access_token", accessToken);
				response.setContentType(APPLICATION_JSON_VALUE);
				new ObjectMapper().writeValue(response.getOutputStream(), tokens);

			} catch (Exception e) {
				log.error("Erro conseguindo refresh token em: " + e.getMessage());
				response.setHeader("error", e.getMessage());
				response.setStatus(FORBIDDEN.value());
				Map<String, String> error = new HashMap<>();
				error.put("error_message", e.getMessage());
				response.setContentType(APPLICATION_JSON_VALUE);
				new ObjectMapper().writeValue(response.getOutputStream(), error);
			}
		} else {
			throw new MissingArgumentsException("Refresh token está faltando", Collections.singletonList("refresh_token"));
		}
	}
}
