package com.example.tcc.service;

import com.example.tcc.document.models.*;
import com.example.tcc.service.exception.*;
import com.example.tcc.service.utils.Tools;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

import static com.example.tcc.service.utils.Constants.*;

@Service
public class WorkService extends AbstractService {

	@Autowired
	private UserService userService;

	@Autowired
	private PersonService personService;

	private Work findByIdWithFields(String id, String... fields) throws WorkNotFoundException {
		Query query = new Query();
		query.addCriteria(Criteria.where(FIELD_ID).is(id))
				.fields().include(fields);
		Work work = db.findOne(query, Work.class);
		if(Objects.isNull(work)) throw new WorkNotFoundException("Obra com id " + id + " não encontrada");
		return work;
	}

	public List<Work> findAll(Integer page) {
		Query query = new Query()
				.with(PageRequest.of(page, 20, Sort.by(FIELD_USER_COUNT, FIELD_RELEASE_YEAR).descending()));
		return db.find(query, Work.class);
	}

	public List<String> findAllTags() {
		return db.findDistinct(FIELD_TAGS, Work.class, String.class);
	}

	public Work findWork(String id) throws WorkNotFoundException {
		return findByIdWithFields(id);
	}

	public List<Work> findMostFamousWorks(String type) {
		Query query = new Query();
		query.with(Sort.by(Sort.Direction.DESC, FIELD_USER_COUNT))
				.limit(HOME_WORKS_MAX)
				.fields().include(FIELD_ID, FIELD_NAME, FIELD_COVER, FIELD_USER_COUNT);
		if(!type.equals(TYPE_ANY))
			query.addCriteria(Criteria.where(FIELD_TYPE).is(type));

		return db.find(query, Work.class);
	}

	public Map<String, List<Work>> findByTags(String type) {
		Map<String, List<Work>> lists = new HashMap<>();
		List<String> tags = findAllTags();
		for(String tag : tags) {
			Query query = new Query().addCriteria(Criteria.where(FIELD_TAGS).is(tag));
			query.limit(HOME_WORKS_MAX)
					.fields().include(FIELD_ID, FIELD_NAME, FIELD_COVER);
			if(!type.equals(TYPE_ANY))
				query.addCriteria(Criteria.where(FIELD_TYPE).is(type));
			List<Work> works = db.find(query, Work.class);
			if(!works.isEmpty())
				lists.put(tag, works);
		}
		return lists;
	}

	public List<Work> searchWorks(String type, String input, Integer page){
		log.info("Initializing work search with input: "+input);
		Query query = new Query();
		query.fields().exclude(FIELD_PEOPLE);
		if(!type.equals(TYPE_ANY))
			query.addCriteria(Criteria.where(FIELD_TYPE).is(type));
		query.addCriteria(new Criteria().orOperator(
				Criteria.where(FIELD_NAME).regex(input, "i"),
				Criteria.where(FIELD_PRODUCTION_COMPANY).regex(input, "i"),
				Criteria.where(FIELD_PUBLISHER).regex(input, "i"),
				Criteria.where(FIELD_DEVELOPER).regex(input, "i")))
				.with(PageRequest.of(page, 20, Sort.by(FIELD_USER_COUNT, FIELD_RELEASE_YEAR).descending()));
		return db.find(query, Work.class);
	}

	public void saveWorkToLoggedUserList(String username, WorkUser newWorkUser) throws WorkNotFoundException, UserNotFoundException {
		Work newWorkUserWork = newWorkUser.getWork();

		try {
			List<WorkUser> loggedUserWorkList = userService.findUserWorkList(username);
			for(WorkUser workUser : loggedUserWorkList)
				if(workUser.getWork().getId().equals(newWorkUserWork.getId())) {
					Query query = new Query().addCriteria(Criteria.where(FIELD_USERNAME).is(username));
					Update update = new Update().pull("workList",
							Query.query(Criteria.where("work._id").is(newWorkUserWork.getId())));
					db.updateFirst(query, update, User.class);
					break;
				}
		} catch (EmptyWorkListException ignored) {
		}

		Work work = findByIdWithFields(newWorkUserWork.getId(),
				FIELD_NAME,
				FIELD_COVER,
				FIELD_TYPE,
				FIELD_RELEASE_YEAR,
				FIELD_EPISODE_NUMBER,
				FIELD_PAGE_NUMBER,
				FIELD_TAGS);
		newWorkUserWork.setName(work.getName());
		newWorkUserWork.setCover(work.getCover());
		newWorkUserWork.setType(work.getType());
		newWorkUserWork.setReleaseYear(work.getReleaseYear());
		newWorkUserWork.setEpisodeNumber(work.getEpisodeNumber());
		newWorkUserWork.setPageNumber(work.getPageNumber());
		newWorkUserWork.setTags(work.getTags());

		if(Objects.isNull(newWorkUser.getStatus())) newWorkUser.setStatus("");
		switch (newWorkUser.getStatus()) {
			case STATUS_COMPLETED:
				switch (newWorkUserWork.getType()) {
					case TYPE_SERIES:
						newWorkUser.setProgress(work.getEpisodeNumber());
						break;
					case TYPE_BOOK:
						newWorkUser.setProgress(work.getPageNumber());
						break;
					default:
						newWorkUser.setProgress(null);
						break;
				}
				break;
			case STATUS_DROPPED:
			case STATUS_ON_HOLD:
			case STATUS_ONGOING:
				break;
			default:
				newWorkUser.setStatus(STATUS_PLANNING);
				newWorkUser.setProgress(0);
				newWorkUser.setScore(null);
		}
		if(Objects.nonNull(newWorkUser.getScore())) {
			if(newWorkUser.getScore() > MAXIMUM_SCORE) newWorkUser.setScore(MAXIMUM_SCORE);
			if(newWorkUser.getScore() < MINIMUM_SCORE) newWorkUser.setScore(MINIMUM_SCORE);
		}

		Query workQuery = new Query();
		workQuery.addCriteria(Criteria.where(FIELD_ID).is(newWorkUserWork.getId()));
		Update workUpdate = new Update();
		workUpdate.inc(FIELD_USER_COUNT);
		db.updateFirst(workQuery, workUpdate, Work.class);

		Query userQuery = new Query();
		userQuery.addCriteria(Criteria.where(FIELD_USERNAME).is(username));
		Update userUpdate = new Update();
		userUpdate.push(FIELD_WORK_LIST, newWorkUser);
		db.updateFirst(userQuery, userUpdate, User.class);
	}

	// ------------------------ Recommendations function START ------------------------
	public Recommendations findRecommendationsForUser(String username, String type) throws UserNotFoundException, EmptyWorkListException {
		List<WorkUser> workList = userService.findUserWorkList(username);
		Query query = new Query();
		query.fields().include(FIELD_ID, FIELD_NAME, FIELD_COVER, FIELD_TAGS);
		if(!TYPE_ANY.equals(type))
			query.addCriteria(Criteria.where(FIELD_TYPE).is(type));
		List<Work> allWorks = db.find(query, Work.class);
		return giveRecommendations(workList, allWorks);
	}

	private Recommendations giveRecommendations(List<WorkUser> userWorks, List<Work> allWorks) {
		log.info("<------- Iniciando algoritmo de recomendações ------->");
		Recommendations recommendations = new Recommendations();
		ArrayList<Work> recomWorks = new ArrayList<>();
		Map<String, Integer> tagScore = new HashMap<>();
		Integer bestTagScore = 0;
		for(WorkUser userWork : userWorks) {
			if(STATUS_COMPLETED.equals(userWork.getStatus())
					&& userWork.getScore() != null) {
				for(String tag : userWork.getWork().getTags()) {
					tagScore.putIfAbsent(tag, 0);
					tagScore.replace(tag, tagScore.get(tag) + userWork.getScore() -3);

					if(tagScore.get(tag) > bestTagScore) bestTagScore = tagScore.get(tag);
				}
			}
			allWorks.removeIf(work -> work.getName().equals(userWork.getWork().getName()));           // remove works already on user's list from all works
		}

		for(Map.Entry<String, Integer> entry : tagScore.entrySet())
			if(entry.getValue() < bestTagScore/2)
				tagScore.replace(entry.getKey(), 0);

		log.info("Notas para cada tag baseada no perfil do usuário atual:");
		log.info(tagScore);

		Map<Work, Integer> workRecomRate = new HashMap<>();
		for(Work work : allWorks) {
			if(Objects.nonNull(work.getTags())){
				for(String tag : work.getTags()) {
					workRecomRate.putIfAbsent(work, 0);
					if(!Objects.isNull(tagScore.get(tag)))
						workRecomRate.replace(work, workRecomRate.get(work) + tagScore.get(tag));       // sets each work's score based on tags and type
				}
				work.setTags(null);
			}
		}

		workRecomRate = sortByComparator(workRecomRate);

		int i = 1;
		for(Map.Entry<Work, Integer> work : workRecomRate.entrySet()) {
			recomWorks.add(work.getKey());
			log.info(work.getKey().getName() + "   " + work.getValue().toString());
			if(i++ >= HOME_WORKS_MAX) break;
		}

		recommendations.setTagScore(tagScore);
		recommendations.setRecommendedWorks(recomWorks);
		return recommendations;
	}

	private static Map<Work, Integer> sortByComparator(Map<Work, Integer> unsortMap) {
		List<Map.Entry<Work, Integer>> list = new LinkedList<>(unsortMap.entrySet());
		list.sort((o1, o2) -> o2.getValue().compareTo(o1.getValue()));

		Map<Work, Integer> sortedMap = new LinkedHashMap<>();
		for(Map.Entry<Work, Integer> entry : list) {
			sortedMap.put(entry.getKey(), entry.getValue());
		}

		return sortedMap;
	}
	// ------------------------ Recommendation functions END ------------------------

	// ------------------------ Saving work functions START ------------------------
	public void saveWork(Work newWork) throws InvalidStatusException, MissingArgumentsException, RepeatedWorkException, MissingRoleException {
		if(Objects.nonNull(newWork.getPeople())) {
			for(PersonWork personWork : newWork.getPeople()) {
				if(Objects.isNull(personWork.getRole()))
					throw new MissingRoleException("Person " + personWork.getPerson().getId() + " doesn't have a role!");
				Person personDb = db.findOne(new Query().addCriteria(Criteria.where(FIELD_ID).is(personWork.getPerson().getId())), Person.class);
				if(Objects.nonNull(personDb)) personWork.getPerson().setName(personDb.getName());
				if(Objects.nonNull(personDb)) personWork.getPerson().setPhoto(personDb.getPhoto());
			}
		}

		try {
			Work repeatedWork = findByIdWithFields(newWork.getId());

			Tools.copyNonNullProperties(repeatedWork, newWork);
			if(newWork.getTags() != null) repeatedWork.setTags(newWork.getTags());
			if(Objects.nonNull(newWork.getPeople())
					&& !newWork.getPeople().isEmpty())
				repeatedWork.setPeople(newWork.getPeople());
			newWork = repeatedWork;
			log.info("Atualizando obra: " + newWork.getName());

			saveWorkOnUsers(newWork);
		} catch (WorkNotFoundException e) {
			validateWork(newWork);
			log.info("Salvando nova obra: " + newWork.getName());
		}

		newWork = db.save(newWork);
		saveWorkOnPeople(newWork);
	}

	private void saveWorkOnUsers(Work newWork) {
		Query query = new Query();
		query.addCriteria(Criteria.where("workList.work._id").is(newWork.getId()));
		Update update = new Update();
		update.set("workList.$.work.name", newWork.getName());
		update.set("workList.$.work.cover", newWork.getCover());
		update.set("workList.$.work.type", newWork.getType());
		update.set("workList.$.work.tags", newWork.getTags());
		if(Objects.nonNull(newWork.getEpisodeNumber()))
			update.set("workList.$.work.episodeNumber", newWork.getEpisodeNumber());
		if(Objects.nonNull(newWork.getPageNumber()))
			update.set("workList.$.work.pageNumber", newWork.getPageNumber());

		db.updateMulti(query, update, User.class);
	}

	private void saveWorkOnPeople(Work newWork) {
		Update update = new Update().pull("workedOn", Query.query(Criteria.where("work._id").is(newWork.getId())));
		db.updateMulti(new Query(), update, Person.class);
		if(Objects.nonNull(newWork.getPeople()))
			for(PersonWork personWork : newWork.getPeople()) {
				Query query = new Query().addCriteria(Criteria
						.where(FIELD_ID).is(personWork.getPerson().getId()));
				PersonWork newPersonWork = new PersonWork();
				newPersonWork.setRole(personWork.getRole());
				newPersonWork.setIsMain(personWork.getIsMain());
				newPersonWork.setPerson(null);
				newPersonWork.setWork(new Work(newWork.getId(),
						newWork.getName(),
						newWork.getCover(),
						newWork.getType()));
				update = new Update().addToSet("workedOn", newPersonWork);
				db.updateFirst(query, update, Person.class);
			}
	}

	private void validateWork(Work newWork) throws InvalidStatusException, MissingArgumentsException, RepeatedWorkException {
		Query query = new Query().addCriteria(Criteria.where(FIELD_NAME).is(newWork.getName()));
		Work repeatedWork = db.findOne(query, Work.class);
		if(Objects.nonNull(repeatedWork))
			throw new RepeatedWorkException("Erro salvando nova obra: Obra com nome " + newWork.getName() + " já existe");

		ArrayList<String> missingArguments = new ArrayList<>();
		if(Objects.isNull(newWork.getName())) missingArguments.add(FIELD_NAME);
		if(Objects.isNull(newWork.getCover())) missingArguments.add(FIELD_COVER);
		if(Objects.isNull(newWork.getPeople()) ||
				newWork.getPeople().isEmpty())
			missingArguments.add(FIELD_PEOPLE);
		if(Objects.isNull(newWork.getType())) missingArguments.add(FIELD_TYPE);
		else {
			switch (newWork.getType()) {
				case TYPE_SERIES:
					if(Objects.isNull(newWork.getEpisodeNumber())) missingArguments.add(FIELD_EPISODE_NUMBER);
					// fallthrough
				case TYPE_MOVIE:
					if(Objects.isNull(newWork.getMinuteDuration())) missingArguments.add(FIELD_MINUTE_DURATION);
					if(Objects.isNull(newWork.getProductionCompany())) missingArguments.add(FIELD_PRODUCTION_COMPANY);
					break;
				case TYPE_BOOK:
					if(Objects.isNull(newWork.getPageNumber())) missingArguments.add(FIELD_PAGE_NUMBER);
					break;
				case TYPE_GAME:
					if(Objects.isNull(newWork.getDeveloper())) missingArguments.add(FIELD_DEVELOPER);
					if(Objects.isNull(newWork.getPublisher())) missingArguments.add(FIELD_PUBLISHER);
					break;
				default:
					throw new InvalidStatusException("Invalid type");
			}
		}
		if(!missingArguments.isEmpty()) {
			log.info("Erro salvando nova obra: Argumentos faltando");
			throw new MissingArgumentsException("Erro salvando nova obra: Argumentos faltando", missingArguments);
		}
	}
	// ------------------------ Saving work functions END ------------------------

	public void calculateWorksAverageScore() {
		Map<String, Double> totalScoresByWorkId = new HashMap<>();
		Map<String, Double> scoreAmountByWorkId = new HashMap<>();

		Query query = new Query();
		query.fields().exclude(FIELD_ID).include(FIELD_WORK_LIST);
		List<User> users = db.find(query, User.class);
		for(User user : users) {
			try {
				for(WorkUser workUser : user.getWorkList()) {
					try {
						if(workUser.getScore() != null) {
							String id = workUser.getWork().getId();
							totalScoresByWorkId.computeIfAbsent(id, t -> {
								scoreAmountByWorkId.put(id, 0D);
								return 0D;
							});
							totalScoresByWorkId.replace(id, totalScoresByWorkId.get(id) + workUser.getScore());
							scoreAmountByWorkId.replace(id, scoreAmountByWorkId.get(id) + 1);
						}
					} catch (NullPointerException ignored) {
						log.info("idk: " + workUser.getWork().getName());
					}
				}
			} catch (NullPointerException ignored) {
				log.info("User without workList: " + user.getName());
			}
		}

		for(Map.Entry<String, Double> total : totalScoresByWorkId.entrySet()) {
			Double avgScore = total.getValue() / scoreAmountByWorkId.get(total.getKey());
			Query query1 = new Query().addCriteria(Criteria.where(FIELD_ID).is(total.getKey()));
			Update update = new Update();
			update.set(FIELD_AVERAGE_SCORE, avgScore);
			db.updateFirst(query1, update, Work.class);
		}
	}

	public void insertWorksByCsv(MultipartFile file) throws IOException {
		Scanner scanner = new Scanner(file.getInputStream());
		scanner.nextLine();

		List<Work> works = new ArrayList<>();
		while(scanner.hasNextLine()) {
			String line = scanner.nextLine();
			List<String> data = Arrays.asList(line.split(","));
			Work work = new Work();

			work.setName(data.get(0));
			if(work.getName() == null) throw new IOException("Nome inválido");

			List<String> tags = Arrays.asList(data.get(1).split(";"));
			work.setTags(tags);
			if(work.getTags() == null
			|| work.getTags().isEmpty()) throw new IOException("Tags inválidas/vazias");

			String type = data.get(2);
			List<String> productionCompanies = Arrays.asList(data.get(5).split(";"));
			switch (type) {
				case "series":
					work.setType(TYPE_SERIES);
					try {
						work.setEpisodeNumber(Integer.valueOf(data.get(3)));
					} catch (NumberFormatException e) {
						e.printStackTrace();
					}
					try {
						work.setMinuteDuration(Integer.valueOf(data.get(6).replaceAll("\\D", "")));
					} catch (NumberFormatException e) {
						log.info("Duração em minutos inválida para obra "+work.getName());
					}
					work.setProductionCompany(productionCompanies.get(0));

					break;
				case "movie":
					work.setType(TYPE_MOVIE);
					try {
						work.setMinuteDuration(Integer.valueOf(data.get(6).replaceAll("\\D", "")));
					} catch (NumberFormatException e) {
						log.info("Duração em minutos inválida para obra "+work.getName());
					}
					work.setProductionCompany(productionCompanies.get(0));
					break;
				case "book":
					work.setType(TYPE_BOOK);
					try {
						work.setPageNumber(Integer.valueOf(data.get(6).replaceAll("\\D", "")));
					} catch (NumberFormatException e) {
						log.info("Número de páginas inválido para obra "+work.getName());
					}
					break;
				case "game":
					work.setType(TYPE_GAME);
					work.setDeveloper(productionCompanies.get(0));
					try{
						work.setPublisher(productionCompanies.get(1));
					} catch (ArrayIndexOutOfBoundsException e) {
						work.setPublisher(productionCompanies.get(0));
					}
					break;
				default:
					throw new IOException("Tipo inválido");
			}

			try{
				work.setReleaseYear(Integer.valueOf(data.get(4).replaceAll("\\D", "")));
				work.setCover(data.get(7));
				work.setSynopsis(data.get(8));
			} catch (ArrayIndexOutOfBoundsException ignored) {
			} catch (NumberFormatException e) {
				log.info("Ano de lançamento inválido para obra "+work.getName());
			}

			works.add(work);
		}
		log.info(works.toString());
		db.insertAll(works);
	}

	public void deleteWork(String id) throws WorkNotFoundException {
		log.info("Deletando obra com id " + id);
		Work work = findByIdWithFields(id);

		List<Person> people = db.find(Query.query(Criteria.where("workedOn.work._id").is(id)), Person.class);
		for (Person person : people) {
			person.getWorkedOn().removeIf(pw -> pw.getWork().getId().equals(id));
			db.save(person);
		}

		db.remove(work);
	}
}
