import 'dart:async';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:tcc_app/blocs/session/session_event.dart';
import 'package:tcc_app/blocs/session/session_state.dart';
import 'package:tcc_app/blocs/user/user_bloc.dart';
import 'package:tcc_app/data/dtos/create_session_dto.dart';
import 'package:tcc_app/data/dtos/profile_dto.dart';
import 'package:tcc_app/services/auth_service.dart';
import 'package:tcc_app/exceptions/app_exception.dart';
import 'package:tcc_app/services/user_service.dart';

class SessionBloc extends Bloc<SessionEvent, SessionState> {
  final AuthService authService;
  final UserService userService;
  final UserBloc userBloc;
  StreamSubscription? _userServiceSubscription;

  SessionBloc({
    required this.authService,
    required this.userService,
    required this.userBloc,
  }) : super(SessionLoadingState()) {
    on<SessionAuthEvent>((event, emit) => _authenticateUser(event, emit));
    on<SessionRevokeAccessEvent>((event, emit) => _logoutUser(emit));
    on<VerifyTokenEvent>((event, emit) => _verifyToken(emit));

    _userServiceSubscription = userBloc.stream.listen((state) {
      if (state is RegisteredUserState) {
        add(
          SessionAuthEvent(
            username: state.username,
            password: state.password,
          ),
        );
      }
    });

    add(VerifyTokenEvent());
  }

  void _authenticateUser(SessionAuthEvent event, Emitter emit) async {
    emit(LoggingUserInState());
    try {
      CreateSessionDto response = await authService.loginWithPassword(
        username: event.username,
        password: event.password,
      );

      final ProfileDto? userProfile = await userService.getUserProfile(username: event.username); 

      emit(SessionReadyState(
        accessToken: response.accessToken,
        username: event.username,
        userProfile: userProfile!,
      ));
    } on AppException catch (e) {
      emit(SessionErrorState(errorMessage: e.message != 'undefined' ? e.message : 'Usuário e/ou senha incorreto(s)'));
    }
  }

  void _logoutUser(Emitter emit) async {
    await authService.logout();

    emit(NoSessionState());
  }

  void _verifyToken(Emitter emit) async {
    try {
      emit(SessionLoadingState());
      final String? userToken = await authService.verifyToken();
      if (userToken != null) {
        final token = JwtDecoder.decode(userToken);
        final String? username = token['sub']?.toString();

        final ProfileDto? userProfile = await userService.getUserProfile(username: username!); 


        emit(SessionReadyState(
          accessToken: userToken,
          username: username,
          userProfile: userProfile!,
        ));
      } else {
        emit(SessionState());
      }
    } on AppException catch (e) {
      emit(SessionErrorState(errorMessage: e.message));
    }
  }

  void _reloadUser(Emitter emit) async {
    try {    
        final ProfileDto? userProfile = await userService.getUserProfile(username: state.username!); 


        emit((state as SessionReadyState).copyWith(
          userProfile: userProfile!,
        ));
    } on AppException catch (e) {
      emit(SessionErrorState(errorMessage: e.message));
    }
  }

  
}
