import 'package:dio/dio.dart';
import 'package:tcc_app/data/repositories/auth_repository.dart';
import 'package:tcc_app/data/repositories/person_repository.dart';
import 'package:tcc_app/data/repositories/user_repository.dart';
import 'package:tcc_app/data/repositories/work_repository.dart';
import 'package:tcc_app/data/repositories/local_device_repository.dart';
import 'package:dio_logger/dio_logger.dart';

class UnitOfWork {
  late AuthRepository _auth;
  late UserRepository _user;
  late WorkRepository _works;
  late PersonRepository _people;
  late Dio _dio;
  static UnitOfWork? _singleton;

  get auth => _auth;
  get user => _user;
  get works => _works;
  get people => _people;
  get dio => _dio;
  get instance => _singleton;

  UnitOfWork({required LocalDeviceRepository localRepository}) {
    if (_singleton == null) {
      BaseOptions options =
          BaseOptions(baseUrl: 'https://tcc-b-e.herokuapp.com');
      _dio = Dio(options);
      _dio.interceptors.add(dioLoggerInterceptor);

      _dio.interceptors.add(InterceptorsWrapper(onRequest:
          (RequestOptions options, RequestInterceptorHandler handler) async {
        if (!options.path.contains('/public')) {
          final bearerToken = await localRepository.getBearerToken();
          options.headers['Authorization'] = "Bearer $bearerToken";
        }
        return handler.next(options);
      }, onError: (DioError error, ErrorInterceptorHandler handler) async {
        if (error.response?.statusCode == 403 && await localRepository.containsKey(REFRESH_TOKEN_KEY)) {
          await _refreshToken(localRepository, _dio);
          return handler.resolve(await _retry(error.requestOptions, _dio));
        }
        return handler.next(error);
      }));

      _auth = AuthRepository(dio: _dio);
      _user = UserRepository(dio: _dio);
      _works = WorkRepository(dio: _dio);
      _people = PersonRepository(dio: dio);
    } else {
      throw Exception(
          "An instance of UnitOfWork is already available. Use UnitOfWork.instance");
    }
  }
}

Future<Response<dynamic>> _retry(RequestOptions requestOptions, Dio dio) async {
  final options = Options(
    method: requestOptions.method,
    headers: requestOptions.headers,
  );
  return dio.request<dynamic>(requestOptions.path,
      data: requestOptions.data,
      queryParameters: requestOptions.queryParameters,
      options: options);
}

Future<void> _refreshToken(LocalDeviceRepository localRepository, Dio dio) async {
  final String? refreshToken = await localRepository.getRefreshToken();
  final response = await dio.get('/public/token/refresh',
      options: Options(
          contentType: Headers.jsonContentType,
          headers: {"Authorization": "Bearer $refreshToken"}));
  if (response.statusCode == 200) {
    localRepository.setBearerToken(response.data['access_token']);
  }
}
