import 'dart:io';

import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:tcc_app/constants/enums.dart';
import 'package:tcc_app/data/dtos/profile_dto.dart';
import 'package:tcc_app/data/dtos/work_in_personal_list_dto.dart';
import 'package:tcc_app/exceptions/app_exception.dart';
import 'package:tcc_app/services/user_service.dart';
import 'package:tcc_app/services/work_service.dart';

part 'edit_profile_event.dart';
part 'edit_profile_state.dart';

class EditProfileBloc extends Bloc<EditProfileEvent, EditProfileState> {
  final UserService userService;
  final WorkService workService;

  EditProfileBloc({
    required this.userService,
    required this.workService,
    required ProfileDto? userProfile,
  }) : super(EditProfileState(
            oldUserProfile: userProfile, editedUserProfile: userProfile)) {
    on<SaveEditProfileEvent>((event, emit) => _editProfile(emit));
    on<ToggleCycleFavoritesEvent>(
        (event, emit) => _toggleCoverOption(event, emit));
    on<SelectCoverImageEvent>((event, emit) => _selectCoverImage(event, emit));
    on<LoadPersonalListEvent>((event, emit) => _loadPersonalList(emit));
    on<ChangeNameEvent>((event, emit) => _changeName(event, emit));
    on<ChangeProfilePictureEvent>(
        (event, emit) => _changeProfilePic(event, emit));
  }

  void _editProfile(Emitter<EditProfileState> emit) async {
    if (state.editedUserProfile?.name == '') return;

    try {
      emit(state.copyWith(isLoading: true));

      if (state.oldUserProfile != state.editedUserProfile) {
        await userService.editUserProfile(profile: state.editedUserProfile!);
      }

      emit(EditProfileSaved());
    } on AppException catch (e) {
    } finally {
      emit(state.copyWith(isLoading: false));
    }
  }

  void _toggleCoverOption(
      ToggleCycleFavoritesEvent event, Emitter<EditProfileState> emit) {
    final ProfileDto? newProfile = state.editedUserProfile?.copyWith(
      coverOption:
          state.editedUserProfile?.coverOption == CoverOption.cycleFavorites
              ? CoverOption.choose
              : CoverOption.cycleFavorites,
    );

    emit(state.copyWith(editedUserProfile: newProfile));
  }

  void _selectCoverImage(
      SelectCoverImageEvent event, Emitter<EditProfileState> emit) {
    final ProfileDto? newProfile = state.editedUserProfile?.copyWith(
      coverOption: CoverOption.choose,
      profileCover: event.coverUrl,
    );

    emit(state.copyWith(editedUserProfile: newProfile));
  }

  void _changeName(ChangeNameEvent event, Emitter<EditProfileState> emit) {
    final ProfileDto? newProfile = state.editedUserProfile?.copyWith(
      name: event.name,
    );

    emit(state.copyWith(editedUserProfile: newProfile));
  }

  void _changeProfilePic(
      ChangeProfilePictureEvent event, Emitter<EditProfileState> emit) async {
    final ProfileDto? newProfile = state.editedUserProfile?.copyWith(
      photo: await event.picture.readAsBytes(),
    );

    emit(state.copyWith(editedUserProfile: newProfile));
  }

  void _loadPersonalList(Emitter<EditProfileState> emit) async {
    emit(state.copyWith(isLoading: true));
    try {
      final workList = await workService
          .getUserPersonalList(state.editedUserProfile!.username);
      if (workList != null) {
        emit(state.copyWith(works: workList));
      }
    } on AppException catch (e) {
    } finally {
      emit(state.copyWith(isLoading: false));
    }
  }
}
