import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:tcc_app/data/dtos/work_in_personal_list_dto.dart';
import 'package:tcc_app/i18n_consts.dart';
import 'package:tcc_app/screens/personal_list/bloc/personal_list_bloc.dart';
import 'package:tcc_app/screens/personal_list/widgets/filter_button.dart';
import 'package:tcc_app/screens/personal_list/widgets/list_grid_toggle.dart';
import 'package:tcc_app/screens/personal_list/widgets/personal_grid.dart';
import 'package:tcc_app/screens/personal_list/widgets/personal_lines.dart';

enum _ListType { grid, list }

enum _SortType { score, name, year }

class PersonalListScreen extends StatefulWidget {
  const PersonalListScreen({Key? key}) : super(key: key);

  @override
  State<PersonalListScreen> createState() => _PersonalListScreenState();
}

class _PersonalListScreenState extends State<PersonalListScreen> {
  _ListType _listType = _ListType.grid;
  String? _statusFilter;
  List<bool> _typeFilter = [true, true, true, true];
  _SortType _sortType = _SortType.score;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text(
          'Minha Lista',
          style: TextStyle(
            fontSize: 36,
            letterSpacing: 4,
            fontWeight: FontWeight.bold,
            color: Colors.white,
          ),
        ),
        centerTitle: true,
        backgroundColor: Colors.black26,
        shadowColor: Colors.transparent,
      ),
      body: BlocBuilder<PersonalListBloc, PersonalListState>(
        builder: (ctx, state) {
          if (state is InitialPersonalListState) {
            context.read<PersonalListBloc>().add(LoadPersonalListEvent());
          }
          if (state is LoadingPersonalListState || state is InitialPersonalListState) {
            return Container(alignment: AlignmentDirectional.center, child: const CircularProgressIndicator());
          }
          if (state is EmptyPersonalListState) {
            return const Center(child: Text('Não há nenhuma obra na sua lista. Adicione algumas!'));
          }
          if (state is SuccessPersonalListState) {
            var works = state.works;

            if (_statusFilter != null) {
              works = works.where((work) => work.status == _statusFilter).toList();
            }

            if (!_typeFilter.every((x) => x == true) && !_typeFilter.every((x) => x == false)) {
              final movies = _typeFilter[0];
              final series = _typeFilter[1];
              final books = _typeFilter[2];
              final games = _typeFilter[3];

              var temp = <WorkInPersonalListDto>[];

              if (movies) {
                temp.addAll(works.where((work) => work.work.type == 'movie'));
              }
              if (series) {
                temp.addAll(works.where((work) => work.work.type == 'series'));
              }
              if (books) {
                temp.addAll(works.where((work) => work.work.type == 'book'));
              }
              if (games) {
                temp.addAll(works.where((work) => work.work.type == 'game'));
              }

              works = temp;
            }

            switch (_sortType) {
              case _SortType.score:
                works.sort((a, b) => b.score.compareTo(a.score), );
                break;
              case _SortType.name:
                works.sort((a, b) => a.work.name.compareTo(b.work.name));
                break;
              case _SortType.year:
                works.sort((a, b) => b.work.releaseYear.compareTo(a.work.releaseYear));
                break;
            }

            return SingleChildScrollView(
              physics: const BouncingScrollPhysics(),
              child: Column(
                children: [
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Padding(
                          padding: const EdgeInsets.symmetric(horizontal: 16.0),
                          child: InkWell(
                            onTap: () {
                              showModalBottomSheet(
                                shape: RoundedRectangleBorder(
                                  borderRadius: BorderRadius.circular(20),
                                ),
                                context: ctx,
                                builder: (builderContext) {
                                  return StatefulBuilder(builder: (BuildContext sctx, StateSetter setModalState) {
                                    void changeTypeSelection(int index) {
                                      final value = !_typeFilter[index];
                                      _setStates(setState, setModalState, () {
                                        _typeFilter[index] = value;
                                      });
                                    }

                                    return Padding(
                                      padding: const EdgeInsets.symmetric(
                                        horizontal: 20.0,
                                        vertical: 24.0,
                                      ),
                                      child: Column(
                                        crossAxisAlignment: CrossAxisAlignment.start,
                                        children: [
                                          const Text(
                                            'Filtro',
                                            style: TextStyle(
                                              fontWeight: FontWeight.bold,
                                              fontSize: 32,
                                            ),
                                          ),
                                          Align(
                                            alignment: Alignment.center,
                                            child: Padding(
                                              padding: const EdgeInsets.all(8.0),
                                              child: DropdownButton<String?>(
                                                value: _statusFilter,
                                                onChanged: (item) {
                                                  _setStates(setState, setModalState, () {
                                                    _statusFilter = item;
                                                  });
                                                },
                                                items: [null, 'ongoing', 'planning', 'completed', 'onhold', 'dropped'].map<DropdownMenuItem<String?>>((String? item) {
                                                  var type = item != null ? (statusName[item] ?? item) : null;
                                                  return DropdownMenuItem<String?>(value: item, child: Text(type != null ? '${type[0].toUpperCase()}${type.substring(1)}' : 'Todos'));
                                                }).toList(),
                                              ),
                                            ),
                                          ),
                                          const Divider(),
                                          Padding(
                                            padding: const EdgeInsets.all(8.0),
                                            child: Row(
                                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                              children: [
                                                FilterButton(text: 'Filmes', active: _typeFilter[0], onTap: () => changeTypeSelection(0)),
                                                FilterButton(text: 'Séries', active: _typeFilter[1], onTap: () => changeTypeSelection(1)),
                                                FilterButton(text: 'Livros', active: _typeFilter[2], onTap: () => changeTypeSelection(2)),
                                                FilterButton(text: 'Jogos', active: _typeFilter[3], onTap: () => changeTypeSelection(3)),
                                              ],
                                            ),
                                          ),
                                          const Divider(),
                                          const Padding(
                                            padding: EdgeInsets.only(top: 32.0),
                                            child: Text(
                                              'Ordenar',
                                              style: TextStyle(
                                                fontWeight: FontWeight.bold,
                                                fontSize: 32,
                                              ),
                                            ),
                                          ),
                                          Row(
                                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                            children: [
                                              SizedBox(
                                                width: 110,
                                                child: RadioListTile<_SortType>(
                                                  title: const Text('Nota'),
                                                  value: _SortType.score,
                                                  groupValue: _sortType,
                                                  contentPadding: const EdgeInsets.all(0),
                                                  activeColor: Colors.white,
                                                  onChanged: (_SortType? value) {
                                                    _setStates(setState, setModalState, () {
                                                      _sortType = value!;
                                                    });
                                                  },
                                                ),
                                              ),
                                              SizedBox(
                                                width: 110,
                                                child: RadioListTile<_SortType>(
                                                  title: const Text('Nome'),
                                                  value: _SortType.name,
                                                  groupValue: _sortType,
                                                  contentPadding: const EdgeInsets.all(0),
                                                  activeColor: Colors.white,
                                                  onChanged: (_SortType? value) {
                                                    _setStates(setState, setModalState, () {
                                                      _sortType = value!;
                                                    });
                                                  },
                                                ),
                                              ),
                                              SizedBox(
                                                width: 110,
                                                child: RadioListTile<_SortType>(
                                                  title: const Text('Ano'),
                                                  value: _SortType.year,
                                                  groupValue: _sortType,
                                                  contentPadding: const EdgeInsets.all(0),
                                                  activeColor: Colors.white,
                                                  onChanged: (_SortType? value) {
                                                    _setStates(setState, setModalState, () {
                                                      _sortType = value!;
                                                    });
                                                  },
                                                ),
                                              ),
                                            ],
                                          ),
                                        ],
                                      ),
                                    );
                                  });
                                },
                              );
                            },
                            child: const Icon(Icons.filter_list),
                          ),
                        ),
                        ListGridToggle(
                          listTypeIndex: _listType == _ListType.grid ? 0 : 1,
                          onToggle: (index) => setState(() {
                            _listType = index == 0 ? _ListType.grid : _ListType.list;
                          }),
                        ),
                      ],
                    ),
                  ),
                  if (_listType == _ListType.grid) PersonalGrid(works: works) else PersonalLines(works: works),
                ],
              ),
            );
          }
          return Container();
        },
      ),
    );
  }

  void _setStates(StateSetter state, StateSetter modalState, void Function() setState) {
    modalState(setState);
    state(setState);
  }
}
