import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:tcc_app/data/dtos/work_status_with_details_dto.dart';
import 'package:tcc_app/data/dtos/work_user_status_dto.dart';
import 'package:tcc_app/i18n_consts.dart';
import 'package:tcc_app/screens/personal_work_status/bloc/personal_work_status_bloc.dart';
import 'package:tcc_app/services/auth_service.dart';
import 'package:tcc_app/services/work_service.dart';
import 'package:tcc_app/widgets/star_input.dart';

class PersonalWorkStatusModal extends StatefulWidget {
  final String workId;

  const PersonalWorkStatusModal(this.workId, {Key? key}) : super(key: key);

  @override
  State<PersonalWorkStatusModal> createState() => _PersonalWorkStatusModalState();
}

class _PersonalWorkStatusModalState extends State<PersonalWorkStatusModal> {
  String? statusSelect;
  int score = 0;
  int? progress;
  bool loaded = false;
  late final TextEditingController _progressController;

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (context) => PersonalWorkStatusBloc(
        authService: AuthService.instance,
        workService: WorkService.instance,
      ),
      child: BlocBuilder<PersonalWorkStatusBloc, PersonalWorkStatusState>(builder: (blocCtx, state) {
        final bloc = blocCtx.read<PersonalWorkStatusBloc>();
        if (state is InitialPersonalWorkStatusState) {
          bloc.add(LoadPersonalWorkStatusEvent(widget.workId));
        }
    
        if (state is InitialPersonalWorkStatusState || state is LoadingPersonalWorkStatusState) {
          return const Center(child: CircularProgressIndicator());
        }
    
        if (state is SuccessPersonalWorkStatusState) {
          if (!loaded) {
            bloc.add(UpdateScreenPersonalWorkStatusEvent(
              () => setState(() {
                statusSelect = state.work.status == '' ? null : state.work.status;
                score = state.work.score;
                progress = state.work.progress;
                loaded = true;
                _progressController = TextEditingController(text: progress?.toString());
              }),
            ));
            return const Center(child: CircularProgressIndicator());
          }
    
          final name = state.work.name;
          final limit = state.work.limit;
    
          return Padding(
            padding: MediaQuery.of(context).viewInsets.copyWith(left: 8.0, right: 8.0),
            child: SingleChildScrollView(
              physics: const BouncingScrollPhysics(),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  const SizedBox(height: 12.0),
                  Text(
                    name,
                    style: const TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 24,
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(vertical:5.0, horizontal: 15.0),
                    child: DropdownButtonFormField<String>(
                      value: statusSelect,
                      hint: Text('Selecione status'),
                      onChanged: (item) {
                        setState(() {
                          statusSelect = item;
                          if (item == 'planning') score = 0;
                          if (item == 'completed' && limit != null) {
                            progress = limit;
                            _progressController.text = limit.toString();
                          }
                        });
                      },
                      items: [ 'ongoing', 'planning', 'completed', 'onhold', 'dropped'].map<DropdownMenuItem<String>>((String item) {
                        var type =  (statusName[item] ?? item);
                        return DropdownMenuItem<String>(value: item, child: Text('${type[0].toUpperCase()}${type.substring(1)}'));
                      }).toList(),    
                      decoration: InputDecoration(
                        labelText: 'Status',
                      ),                      
                    ),
                  ),
                  if (limit != null)
                    const Divider(
                      thickness: 2,
                      color: Colors.white24,
                    ),
                  if (limit != null)
                    Wrap(alignment: WrapAlignment.center, children: [
                      const Padding(
                        padding: EdgeInsets.only(bottom: 2, top: 8),
                        child: Text(
                          'Progresso',
                          textAlign: TextAlign.center,
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.all(2.0),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          children: [
                            SizedBox(
                              width: 50,
                              child: TextFormField(
                                controller: _progressController,
                                onChanged: (value) {
                                  setState(() => progress = int.tryParse(value));
                                  
                                  if (progress != null) {
                                    if (progress! > limit) {
                                      setState(() => progress = limit);
                                      _progressController.text = limit.toString();
                                      _progressController.selection = TextSelection.fromPosition(TextPosition(offset: _progressController.text.length));
                                    }
                                  }
                                },
                                readOnly: statusSelect == 'completed',
                                keyboardType: TextInputType.number,
                                decoration: InputDecoration(
                                  isCollapsed: true,
                                  contentPadding: EdgeInsets.only(bottom: 7.0),
                                )
                              ),
                            ),
                            const Text('/'),
                            Text(limit.toString()),
                          ],
                        ),
                      ),
                    ]),
                  const Divider(
                    thickness: 2,
                    color: Colors.white24,
                  ),
                  if (statusSelect != 'planning')
                    Wrap(
                      alignment: WrapAlignment.center,
                      children: [
                        const Text('Avaliação'),
                        Padding(
                          padding: const EdgeInsets.all(2.0),
                          child: Align(
                            alignment: Alignment.center,
                            child: StarInput(
                              rating: score.toDouble(),
                              setRating: (rating) => setState(() => score = rating),
                              size: 48,
                            ),
                          ),
                        ),
                      ],
                    ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: InkWell(
                      onTap: () {
                        if (progress != null && progress! > limit!) {
                          setState(() {
                            progress = limit;
                          });
                        }
                
                        if (statusSelect != null) {
                          var work = WorkUserStatusDto(
                            status: statusSelect!,
                            score: score,
                            progress: progress,
                          );
                
                          bloc.add(
                            SavePersonalWorkStatusEvent(
                              currentState: SuccessPersonalWorkStatusState(
                                work: WorkStatusWithDetailsDto(
                                  id: widget.workId,
                                  name: name,
                                  status: statusSelect ?? '',
                                  score: score,
                                  limit: limit,
                                  progress: progress,
                                ),
                              ),
                              workId: widget.workId,
                              work: work,
                              callback: (result) {
                                if (result) Navigator.pop(context);
                              },
                            ),
                          );
                        }
                      },
                      child: Container(
                        decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(32),
                          border: Border.all(color: Colors.white, width: 4),
                        ),
                        child: const Padding(
                          padding: EdgeInsets.symmetric(horizontal: 32.0, vertical: 12.0),
                          child: Text(
                            'Adicionar à Minha Lista',
                            style: TextStyle(fontSize: 22),
                          ),
                        ),
                      ),
                    ),
                  ),
                  const SizedBox(height: 12.0),

                ],
              ),
            ),
          );
        }
    
        return Container();
      }),
    );
  }
}
