import 'package:flutter/material.dart';
import 'package:mask_text_input_formatter/mask_text_input_formatter.dart';
import 'package:tcc_app/blocs/user/user_bloc.dart';
import 'package:tcc_app/helpers/string.dart';
import 'package:tcc_app/screens/login/widgets/login_container.dart';
import 'package:tcc_app/widgets/main_button.dart';
import 'package:tcc_app/widgets/text_form_input.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class RegisterScreen extends StatefulWidget {

  const RegisterScreen({Key? key}) : super(key: key);

  @override
  State<RegisterScreen> createState() => _RegisterScreenState();
}

class _RegisterScreenState extends State<RegisterScreen> {
  final _form = GlobalKey<FormState>();

  final MaskTextInputFormatter dateMaskFormatter = MaskTextInputFormatter(
      mask: '##/##/####',
      filter: {"#": RegExp(r'[0-9]')},
      type: MaskAutoCompletionType.lazy);

  final TextEditingController nameController = TextEditingController();
  final TextEditingController usernameController = TextEditingController();
  final TextEditingController emailController = TextEditingController();
  final TextEditingController birthDateController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();
  final TextEditingController repeatPasswordController = TextEditingController();

  String? emailError;
  String? userNameError;
  bool isLoading = false;

  void _save(BuildContext context) {
    setState(() => isLoading = true);
    if (!_form.currentState!.validate()) {
      setState(() => isLoading = false);
      return;
    }

    context.read<UserBloc>().add(RegisterUserEvent(
          name: nameController.text,
          username: usernameController.text,
          email: emailController.text,
          password: passwordController.text,
          birthDate: birthDateController.text,
          onError:(message) => setState((){
            if (message.contains('email')) {
              emailError = message.replaceFirst('Erro ao salvar usuário: ', '');
            }
            if (message.contains('username')) {
              userNameError = message.replaceFirst('Erro ao salvar usuário: ', '');
            }
            !_form.currentState!.validate();
            isLoading = false;
          }),
        ));
    setState(() => isLoading = false);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Container(
          alignment: Alignment.center,
          height: double.infinity,
          decoration: const BoxDecoration(
            image: DecorationImage(
              image: AssetImage(
                'assets/images/background_image.png',
              ),
              fit: BoxFit.fill,
            ),
          ),
          child: SingleChildScrollView(
            physics: const BouncingScrollPhysics(),
            child: LoginContainer(
              child: Form(
                key: _form,
                child: BlocBuilder<UserBloc, UserState>(
                  builder: (context, state) {
                    return Column(
                      children: [
                        const Text(
                          'Cadastro',
                          style: TextStyle(
                              color: Colors.white,
                              fontSize: 48.0,
                              fontWeight: FontWeight.bold,
                              letterSpacing: 5.0),
                        ),
                        const SizedBox(height: 50.0),
                        TextFormInput(
                          controller: nameController,
                          enabled: state is! LoadingUserState,
                          hintText: 'nome',
                          textInputAction: TextInputAction.next,
                        ),
                        const SizedBox(height: 30.0),
                        TextFormInput(
                          controller: usernameController,
                          enabled: state is! LoadingUserState,
                          hintText: 'nome de usuário',
                          textInputAction: TextInputAction.next,
                          validator: (value) {
                            if (value == null || value == '') {
                              return 'Preencha o nome de usuário';
                            }

                            if (userNameError != null) {
                              return userNameError;
                            }

                            return null;
                          },
                          onChanged: (_) {
                            if (userNameError != null) {
                              setState(() {
                                userNameError = null;
                              });
                            }
                          },
                        ),
                        const SizedBox(height: 30.0),
                        TextFormInput(
                          controller: emailController,
                          enabled: state is! LoadingUserState,
                          hintText: 'email',
                          textInputAction: TextInputAction.next,
                          keyboardType: TextInputType.emailAddress,
                          validator: (value) {
                            if (value == null || value == '') {
                              return 'preencha o e-mail';
                            }

                            if (emailError != null) {
                              return emailError;
                            }

                            if (!RegExp(
                                    r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+")
                                .hasMatch(value)) return 'e-mail inválido';

                            return null;
                          },
                          onChanged: (_) {
                            if (emailError != null) {
                              setState(() {
                                emailError = null;
                              });
                            }
                          },
                        ),
                        const SizedBox(height: 30.0),
                        TextFormInput(
                          controller: birthDateController,
                          enabled: state is! LoadingUserState,
                          hintText: 'data de nasc.',
                          keyboardType: TextInputType.number,
                          textInputAction: TextInputAction.next,
                          formatters: [dateMaskFormatter],
                          validator: (value) {
                            if (value == null || value == '') {
                              return 'preencha a data de nascimento';
                            }
                            if (!isOlderThan13(value)) {
                              return 'idade minima não atingida';
                            }
                            return null;
                          },
                        ),
                        const SizedBox(height: 30.0),
                        TextFormInput(
                            controller: passwordController,
                            textInputAction: TextInputAction.next,
                            enabled: state is! LoadingUserState,
                            hintText: 'senha',
                            obscureText: true,
                            validator: (value) {
                              if (value == null || value == '') {
                                return 'preencha a senha';
                              }
                              if (value.length < 8) {
                                return 'a senha deve conter no mínimo 8 dígitos';
                              }
                              if (value != repeatPasswordController.text) {
                                return 'senhas não coincidem';
                              }
                              return null;
                            }),
                        const SizedBox(height: 30.0),
                        TextFormInput(
                            controller: repeatPasswordController,
                            enabled: state is! LoadingUserState,
                            hintText: 'repita a senha',
                            obscureText: true,
                            validator: (value) {
                              if (value == null || value == '') {
                                return 'preencha a senha';
                              }
                              if (value.length < 8) {
                                return 'a senha deve conter no mínimo 8 dígitos';
                              }
                              if (value != passwordController.text) {
                                return 'senhas não coincidem';
                              }
                              return null;
                            }),
                        const SizedBox(height: 50.0),
                        MainButton(
                          loading: state is LoadingUserState || isLoading,
                          child: const Text('Entrar'),
                          onTap: () => _save(context),
                        ),
                      ],
                    );
                  },
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
