import 'package:dio/dio.dart';
import 'package:tcc_app/data/dtos/read_work_details_dto.dart';
import 'package:tcc_app/data/dtos/work_in_personal_list_dto.dart';
import 'package:tcc_app/data/dtos/work_user_status_dto.dart';
import 'package:tcc_app/view_model/work_view_model.dart';
import 'package:tcc_app/data/repositories/work_repository.dart';

class WorkService {
  static WorkService? _singleton;

  static get instance => _singleton;

  final WorkRepository workRepository;

  WorkService({required this.workRepository}) {
    if (_singleton == null) {
      _singleton = this;
    } else {
      throw Exception(
          'An instance of WorkService is already available. Use WorkService.instance.');
    }
  }

  Future<List<WorkViewModel>?>? getRecommendations({String? type}) async {
    List<WorkViewModel>? workList;

    Response? response =
        await workRepository.fetchRecommendationFromDataBase(type: type);

    if (response != null && response.data != null && response.data.isEmpty) return [];

    workList = response!.data['recommendedWorks'].map<WorkViewModel>((work) {
      return WorkViewModel.fromResponseContent(work);
    }).toList();

    return workList;
  }

  Future<List<WorkViewModel>?>? getFamousWorks({String? type}) async {
    List<WorkViewModel>? workList;

    Response? response =
        await workRepository.fetchFamousWorksFromDataBase(type: type);

    workList = response!.data.map<WorkViewModel>((work) {
      return WorkViewModel.fromResponseContent(work);
    }).toList();

    return workList;
  }

  Future<Map<String, List<WorkViewModel>>?>? getWorksByTags(
      {String? type}) async {
    Map<String, List<WorkViewModel>>? worksByTags = Map();

    Response? response =
        await workRepository.fetchWorksByTagsFromDataBase(type: type);

    response!.data.forEach((key, value) {
      worksByTags[key] = value
          .map<WorkViewModel>((work) => WorkViewModel.fromResponseContent(work))
          .toList();
    });

    return worksByTags;
  }

  Future<ReadWorkDetailsDto?> findById(String workId) async {
    Response? response = await workRepository.fetchWorkById(workId);

    return ReadWorkDetailsDto.fromResponseContent(response!.data);
  }

  Future<WorkUserStatusDto?> getUserPersonalWorkStatus(
      String username, String workId) async {
    try {
      Response? response =
          await workRepository.fetchWorkInUserList(username, workId);

      if (response == null || response.data == null) {
        return null;
      }

      return WorkUserStatusDto.fromResponseContent(response.data);
    } catch (_) {
      return null;
    }
  }

  Future<bool> addOrUpdatePersonalWorkStatus(
      String username, String workId, WorkUserStatusDto work) async {
    final planningOrWithScore = work.status == 'planning' || (work.score != null && work.score! > 0);
    if (username.isEmpty || workId.isEmpty || work.status.isEmpty || !planningOrWithScore) {
      return false;
    }
    Response? response =
        await workRepository.upsertWorkInUserList(username, workId, work);
    return response?.statusCode == 200;
  }

  Future<List<WorkInPersonalListDto>?> getUserPersonalList(
      String username) async {
    var response = await workRepository.fetchUserPersonalList(username);

    if (response != null && (response.data == null || response.data.isEmpty)) return [];

    return response!.data
        .map<WorkInPersonalListDto>(
            (workJson) => WorkInPersonalListDto.fromResponseContent(workJson))
        .toList();
  }

  Future<List<ReadWorkDetailsDto>?> searchWork(
      {required String query, required String type}) async {
    Response? response =
        await workRepository.fetchWorkByQuery(query: query, type: type);

    final List<ReadWorkDetailsDto> works =
        response!.data.map<ReadWorkDetailsDto>((e) {
      return ReadWorkDetailsDto.fromResponseContent(e);
    }).toList();

    return works;
  }
}
