import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:tcc_app/data/dtos/login_request_dto.dart';
import 'package:tcc_app/services/auth_service.dart';
import 'package:tcc_app/exceptions/app_exception.dart';
import 'package:tcc_app/data/repositories/auth_repository.dart';
import 'package:tcc_app/data/repositories/local_device_repository.dart';

class AuthRepositoryMock extends Mock implements AuthRepository {}

class LocalDeviceRepositoryMock extends Mock implements LocalDeviceRepository {}

void main() {
  late AuthRepository authRepository;
  late LocalDeviceRepository localRepository;
  late AuthService sut;

  setUpAll(() {
    authRepository = AuthRepositoryMock();
    localRepository = LocalDeviceRepositoryMock();
    sut = AuthService(
      authRepository: authRepository,
      localRepository: localRepository,
    );
  });

  final validLoginParams = _makeValidLoginParams();

  final invalidLoginParams = _makeInvalidLoginParams();

  group('Login |', () {
    test(
      "given valid and existing user data, when it calls 'loginWithPassword', then it should login",
      () async {
        when(() => authRepository.login(params: validLoginParams))
            .thenAnswer((_) async => _makeResponse());

        expect(
            () => sut.loginWithPassword(
                username: 'valid_username', password: 'valid_password'),
            returnsNormally);

        verify(() => authRepository.login(params: validLoginParams)).called(1);
      },
    );

    test(
      "given invalid or inexisting user data, when it calls 'loginWithPassword', then it should fail to login",
      () async {
        when(() => authRepository.login(params: invalidLoginParams))
            .thenThrow(AppException('XX', 'Login failed'));

        expect(
          () => sut.loginWithPassword(
              username: 'invalid_username', password: 'invalid_password'),
          throwsException,
        );

        verify(() => authRepository.login(params: invalidLoginParams))
            .called(1);
      },
    );
  });

  group('Logout |', () {
    test(
      "given a logged user, when it calls 'logout', then it should logout by cleaning the local device repository data",
      () async {
        when(() => localRepository.clear()).thenAnswer((_) => null);

        expect(() => sut.logout(), returnsNormally);

        verify(() => localRepository.clear()).called(1);
      },
    );
  });
}

LoginRequestDto _makeValidLoginParams() {
  return const LoginRequestDto(
    username: 'valid_username',
    password: 'valid_password'
  );
}

LoginRequestDto _makeInvalidLoginParams() {
  return const LoginRequestDto(
    username: 'invalid_username',
    password:'invalid_password'
    );
}

Response _makeResponse() {
  return Response(
    statusCode: 200,
    data: {"access_token": "1234", "refresh_token": "12345"},
    requestOptions: RequestOptions(
      path: 'http://localhost:8080/public/user/save',
    ),
  );
}
